// Copyright (c) 2015, Will Squire. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library functional_behaviour_tree.test.node;

import 'package:test/test.dart';
import 'package:functional_behaviour_tree/functional_behaviour_tree.dart';
import 'dart:async';
import 'blackboards/http_blackboard.dart';
import 'nodes/array_based/isHttpStatus.dart' as http_status_array;
import 'nodes/class_based/isHttpStatus.dart' as http_status_class;

node_test() {
  group('Node with declaration', () {

    Node node;

    setUp(() {
      // Assign function declaration
      node = http_status_array.isHttpStatusCode404;
    });

    test('Can return true', () {
      return node({'http_status_code' : '404'}).then((bool value) { expect(value, isTrue); });
    });

    test('Can return false', () {
      return node({'http_status_code' : '401'}).then((bool value) { expect(value, isFalse); });
    });

  });

  group('Node with expression', () {

    Node node;

    setUp(() {
      // Assign function expression
      node = (blackboard) {
        if (blackboard['http_status_code'] == '404')
          return new Future(() => true);
        return new Future(() => false);
      };
    });

    test('Can return true', () {
      return node({'http_status_code' : '404'}).then((bool value) { expect(value, isTrue); });
    });

    test('Can return false', () {
      return node({'http_status_code' : '401'}).then((bool value) { expect(value, isFalse); });
    });

  });

  group('Node using blackboard as a class', () {

    Blackboard blackboard;
    Node node;

    setUp(() {
      blackboard = new Blackboard();
      blackboard.http_status_code = 404;
      node = http_status_class.isHttpStatusCode404;
    });

    test('Can return true', () {
      return node(blackboard).then((bool value) { expect(value, isTrue); });
    });

    test('Can return true', () {
      return node(blackboard).then((bool value) { expect(value, isTrue); });
    });

  });

  // TODO - Testing for classes, improve documentation. (Can still do array based blackboard, but a class based blackboard has the advantage of need to predefined variables which cut down accessing array entries that might not exist (null pointers)) It does this by requireing the blackboard as an object (which is super class, thus arrays and classes can both be used)
  // ------- Have interfaces for nodes that main blackboard in the tree can implement? Mixin? -----------

}
